#ifndef CAMERACONTROLER_H
#define CAMERACONTROLER_H

#include <QThread>
#include "opencv2/opencv.hpp"
#include "opencv2/objdetect.hpp"
#include <opencv2/face.hpp>

using namespace cv;

class CameraTrainer;
class CameraRecognition;

class CameraControler: public QThread{

    public:
        /**
            @brief Constructor

            Se prende la cámara del equipo y se setean los parámetros necesarios
            para el reconocimiento de rostro. En caso de existir un modelo lo carga
            en memoria. 
        */
        CameraControler();

        /**
            @brief Destructor

            Apaga la cámara del equipo
        */
        ~CameraControler();

        /**
            @brief Pasa del menu principal a la fase de uso de la cámara

            En caso de que el objeto esté en estado de entrenamiento se
            abre la interfaz de registro de un nuevo usuario. En caso
            contrario se abre la interfaz de reconocimiento de rostros.
        */
        void run();

        /**
            @brief Habilitar el refrezco de la imagen por pantalla
            @param estadoCamara Si es necesario o no refrezcar la imagen por pantalla

            Esta función se encarga de habilitar la existencia de un hilo de refrezco de 
            la imagen por pantalla, necesario para poder ver los datos de la camara a través 
            del entrenamiento o de la fase de reconocimiento.
        */
        void setEstadoCamara(bool estadoCamara);

        /**
            @brief Setea la ventana del cameraTrainer
            @param cameraTrainer Puntero a CameraTrainer
        */
        void setCameraTrainer(CameraTrainer *cameraTrainer);

        /**
            @brief Setea la ventana del cameraRecognition
            @param cameraRecognition Puntero a CameraRecognition 
        */
        void setCameraRecognition(CameraRecognition *cameraRecognition);

        /**
            @brief Setea el estado de IamTraining
            @param IamTraining Si es que está en modo de entrenamiento

            Si es que IamTraining es verdadero una vez que se sale del menú principal
            se puede pasar a la fase de entrenamiento. De lo contrario se puede pasar
            a la fase de reconocimiento.
        */
        void setIamTraining(bool IamTraining);

        /**
            @brief Recorte de rostro
            @param face Matriz de pixeles del rostro recortado
            @param warped Matriz de pixeles con el rostro estandarizado para reconocimiento
            @param leftEye Rectángulo con coordenadas de ojo izquierdo
            @param rightEye Rectángulo con coordenadas de ojo derecho

            Esta función devuelve el rostro alineado, recortado y convertido a escala de grises.
	    Esta función fue sacada de: http://acodigo.blogspot.com/2014/03/reconocimiento-facial.html          
        */
        void CropFace(const Mat& face, Mat& warped, Rect leftEye, Rect rightEye);

        /**
            @brief Permite que se pueda capturar una nueva imagen
        */
        void update();

        /**
           @brief Muestra una imagen del video en tiempo de llamado
           @return Retorna la imagen instantánea capturada por la cámara

        */
        VideoCapture *getCap();

    private:
        VideoCapture cap;
        Ptr<cv::face::FaceRecognizer> faceRecognizer;
        bool estadoCamara;
        bool IamTraining;
        bool IsUpdated;
        CameraTrainer *window;
        CameraRecognition *cameraRecognition;
        void Training();
        void Recognizing();
        CascadeClassifier detectorFace;
        CascadeClassifier detectorEye;
        float EYE_SX;
        float EYE_SY;
        float EYE_SW;
        float EYE_SH;
        double DESIRED_LEFT_EYE_Y;
        double DESIRED_LEFT_EYE_X;
        int FaceWidth;
        int FaceHeight;

};

#endif // CAMERACONTROLER_H
