#ifndef CAMERATRAINER_H
#define CAMERATRAINER_H

#include <QDialog>
#include "cameracontroler.h"

namespace Ui {
class CameraTrainer;
}

/**
 * Clase que implementa el comportamiento de CameraTrainer.
 * Es la clase de la ventana donde se realiza la recopilacion de datos (fotos) de la persona.
 * @author Luis Leiva, Felipe Muñoz, Sebastian Ubiergo
 */
class CameraTrainer : public QDialog
{
    Q_OBJECT

public:
    /**
     * Constructor de CameraTrainer. Inicializa un nuevo objeto.
     * @param parent:
     */
    explicit CameraTrainer(QWidget *parent = 0);
    /**
     * Destructor de CameraTrainer.
     */
    ~CameraTrainer();
    /**
     * Metodo para setear una imagen en al label que se muestra la camara.
     * @param imagen: La imagen que se desea setear.
     */
    void setImage(QPixmap *);
    /**
     * Metodo para inicializar el hilo que permite ir actualizando la imagen en pantalla.
     */
    void turnCameraOn();
    /**
     * Metodo para finalizar el hilo que permite ir actualizando la imagen en pantalla.
     */
    void turnCameraOff();
    /**
     * Setea el CameraControler encargado de actualizar la imagen y realizar el guardado de los datos (fotos).
     * @param cameraControler: Encargado de actualizar la imagen y realizar el guardado de los datos (fotos).
     */
    void setCameraControler(CameraControler *);
    /**
     * Setea la variable para indicar si es posible tomar la foto.
     * @param IcanTakePhoto: Si es true, entonces si se puede tomar foto. Si es false no se puede.
     */
    void setIcanTakePhoto(bool);
    /**
     * Setea el Rol para asociar la personas con el numero del rol.
     * @param Rol: Numero con que se va asociar a la persona.
     */
    void setRol(int);
    /**
     * Setea Name para asociar la personas con su nombre. Para futuras implementaciones.
     * @param Name: Nombre de la persona.
     */
    void setName(String);
    /**
     * Retorna el Rol de la persona.
     * @return Retorna el Rol de la persona.
     */
    int getRol();
    /**
     * Retorna el nombre de la persona.
     * @return Retorna el nombre de la persona.
     */
    String getName();
    /**
     * Retorna un puntero a ChangeButton.
     * @return Retorna un puntero a ChangeButton.
     */
    QPushButton *getChangeButton();

public slots:
    /**
     * Redefine la funcion de cierre para que ademas apague la camara.
     */
    void reject();
    /**
     * Setea update de cameraControler.
     */
    void Training();

private:
    Ui::CameraTrainer *ui;
    CameraControler *cameraControler;
    QPalette *Pal;
    bool IcanTakePhoto;
    int Rol;
    String Name;

};

#endif // CAMERATRAINER_H
